@@ -1,12 +1,27 @@
-# scapy.contrib.description = ISIS
+# This file is part of Scapy
+# Scapy is free software: you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation, either version 2 of the License, or
+# any later version.
+#
+# Scapy is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with Scapy. If not, see <http://www.gnu.org/licenses/>.
+
+# scapy.contrib.description = Intermediate System to Intermediate System (ISIS)
 # scapy.contrib.status = loads
 
 """
     IS-IS Scapy Extension
     ~~~~~~~~~~~~~~~~~~~~~
 
-    :copyright: 2014, 2015 BENOCS GmbH, Berlin (Germany)
+    :copyright: 2014-2016 BENOCS GmbH, Berlin (Germany)
     :author:    Marcel Patzlaff, mpatzlaff@benocs.com
+                Michal Kaliszan, mkaliszan@benocs.com
     :license:   GPLv2
 
         This module is free software; you can redistribute it and/or
@@ -29,7 +44,7 @@
          * RFC 1195 (only the TCP/IP related part)
          * RFC 3358 (optional checksums)
          * RFC 5301 (dynamic hostname extension)
-         * RFC 5302 (domain-wide prefix destribution)
+         * RFC 5302 (domain-wide prefix distribution)
          * RFC 5303 (three-way handshake)
          * RFC 5304 (cryptographic authentication)
          * RFC 5308 (routing IPv6 with IS-IS)
@@ -45,26 +60,32 @@
 
 """
 
+from __future__ import absolute_import
 import struct
 import random
 
 from scapy.config import conf
-from scapy.fields import *
-from scapy.packet import *
+from scapy.fields import BitField, BitFieldLenField, BoundStrLenField, \
+    ByteEnumField, ByteField, ConditionalField, Field, FieldLenField, \
+    FieldListField, FlagsField, IEEEFloatField, IP6PrefixField, IPField, \
+    IPPrefixField, IntField, LongField, MACField, PacketListField, \
+    ShortField, ThreeBytesField, XIntField, XShortField
+from scapy.packet import bind_layers, Packet
 from scapy.layers.clns import network_layer_protocol_ids, register_cln_protocol
-from scapy.layers.inet6 import IP6ListField
+from scapy.layers.inet6 import IP6ListField, IP6Field
+from scapy.utils import fletcher16_checkbytes
+from scapy.volatile import RandString, RandByte
+from scapy.modules.six.moves import range
+from scapy.compat import orb, hex_bytes
 
-
-EXT_VERSION = "v0.0.1"
-
-conf.debug_dissector = True
+EXT_VERSION = "v0.0.2"
 
 
 #######################################################################
-##  ISIS Utilities + Fields                                          ##
+#   ISIS Utilities + Fields                                           #
 #######################################################################
 def isis_area2str(area):
-    return "".join(x.decode("hex") for x in area.split("."))
+    return b"".join(hex_bytes(x) for x in area.split("."))
 
 
 def isis_str2area(s):
@@ -72,36 +93,37 @@ def isis_str2area(s):
         return ""
 
     numbytes = len(s[1:])
-    fmt = "%02X" + (".%02X%02X" * (numbytes / 2)) + ("" if (numbytes % 2) == 0 else ".%02X")
-    return fmt % tuple(map(ord, s))
+    fmt = "%02X" + (".%02X%02X" * (numbytes // 2)) + ("" if (numbytes % 2) == 0 else ".%02X")  # noqa: E501
+    return fmt % tuple(orb(x) for x in s)
 
 
 def isis_sysid2str(sysid):
-    return "".join(x.decode("hex") for x in sysid.split("."))
+    return b"".join(hex_bytes(x) for x in sysid.split("."))
 
 
 def isis_str2sysid(s):
-    return ("%02X%02X."*3)[:-1] % tuple(map(ord, s))
+    return ("%02X%02X." * 3)[:-1] % tuple(orb(x) for x in s)
 
 
 def isis_nodeid2str(nodeid):
-    return "%s%s" % (isis_sysid2str(nodeid[:-3]), nodeid[-2:].decode("hex"))
+    return isis_sysid2str(nodeid[:-3]) + hex_bytes(nodeid[-2:])
 
 
 def isis_str2nodeid(s):
-    return "%s.%02X" % (isis_str2sysid(s[:-1]), ord(s[-1]))
+    return "%s.%02X" % (isis_str2sysid(s[:-1]), orb(s[-1]))
 
 
 def isis_lspid2str(lspid):
-    return "%s%s" % (isis_nodeid2str(lspid[:-3]), lspid[-2:].decode("hex"))
+    return isis_nodeid2str(lspid[:-3]) + hex_bytes(lspid[-2:])
 
 
 def isis_str2lspid(s):
-    return "%s-%02X" % (isis_str2nodeid(s[:-1]), ord(s[-1]))
+    return "%s-%02X" % (isis_str2nodeid(s[:-1]), orb(s[-1]))
 
 
 class _ISIS_IdFieldBase(Field):
     __slots__ = ["to_str", "to_id", "length"]
+
     def __init__(self, name, default, length, to_str, to_id):
         self.to_str = to_str
         self.to_id = to_id
@@ -110,7 +132,7 @@ class _ISIS_IdFieldBase(Field):
 
     def i2m(self, pkt, x):
         if x is None:
-            return "\0"*self.length
+            return b"\0" * self.length
 
         return self.to_str(x)
 
@@ -118,7 +140,7 @@ class _ISIS_IdFieldBase(Field):
         return self.to_id(x)
 
     def any2i(self, pkt, x):
-        if type(x) is str and len(x) == self.length:
+        if isinstance(x, str) and len(x) == self.length:
             return self.m2i(pkt, x)
 
         return x
@@ -135,16 +157,16 @@ class _ISIS_RandId(RandString):
 
         val = ()
 
-        for _ in xrange(self.bytecount):
+        for _ in range(self.bytecount):
             val += (RandByte(),)
 
         return self.format % val
 
 
 class _ISIS_RandAreaId(_ISIS_RandId):
-    def __init__(self, bytecount= None):
-        self.bytecount = random.randint(1, 13) if bytecount is None else bytecount
-        self.format = "%02X" + (".%02X%02X" * ((self.bytecount-1) / 2)) + ("" if ((self.bytecount-1) % 2) == 0 else ".%02X")
+    def __init__(self, bytecount=None):
+        self.bytecount = random.randint(1, 13) if bytecount is None else bytecount  # noqa: E501
+        self.format = "%02X" + (".%02X%02X" * ((self.bytecount - 1) // 2)) + ("" if ((self.bytecount - 1) % 2) == 0 else ".%02X")  # noqa: E501
 
 
 class ISIS_AreaIdField(Field):
@@ -163,17 +185,17 @@ class ISIS_AreaIdField(Field):
     def i2len(self, pkt, x):
         if x is None:
             return 0
-        l = len(x)
+        tmp_len = len(x)
         # l/5 is the number of dots in the Area ID
-        return (l - (l / 5)) / 2
+        return (tmp_len - (tmp_len // 5)) // 2
 
     def addfield(self, pkt, s, val):
         sval = self.i2m(pkt, val)
-        return s+struct.pack("!%is" % len(sval), sval)
+        return s + struct.pack("!%is" % len(sval), sval)
 
     def getfield(self, pkt, s):
         numbytes = self.length_from(pkt)
-        return s[numbytes:], self.m2i(pkt, struct.unpack("!%is" % numbytes, s[:numbytes])[0])
+        return s[numbytes:], self.m2i(pkt, struct.unpack("!%is" % numbytes, s[:numbytes])[0])  # noqa: E501
 
     def randval(self):
         return _ISIS_RandAreaId()
@@ -181,7 +203,7 @@ class ISIS_AreaIdField(Field):
 
 class ISIS_SystemIdField(_ISIS_IdFieldBase):
     def __init__(self, name, default):
-        _ISIS_IdFieldBase.__init__(self, name, default, 6, isis_sysid2str, isis_str2sysid)
+        _ISIS_IdFieldBase.__init__(self, name, default, 6, isis_sysid2str, isis_str2sysid)  # noqa: E501
 
     def randval(self):
         return _ISIS_RandId("**.**.**")
@@ -189,7 +211,7 @@ class ISIS_SystemIdField(_ISIS_IdFieldBase):
 
 class ISIS_NodeIdField(_ISIS_IdFieldBase):
     def __init__(self, name, default):
-        _ISIS_IdFieldBase.__init__(self, name, default, 7, isis_nodeid2str, isis_str2nodeid)
+        _ISIS_IdFieldBase.__init__(self, name, default, 7, isis_nodeid2str, isis_str2nodeid)  # noqa: E501
 
     def randval(self):
         return _ISIS_RandId("**.**.**.*")
@@ -197,7 +219,7 @@ class ISIS_NodeIdField(_ISIS_IdFieldBase):
 
 class ISIS_LspIdField(_ISIS_IdFieldBase):
     def __init__(self, name, default):
-        _ISIS_IdFieldBase.__init__(self, name, default, 8, isis_lspid2str, isis_str2lspid)
+        _ISIS_IdFieldBase.__init__(self, name, default, 8, isis_lspid2str, isis_str2lspid)  # noqa: E501
 
     def randval(self):
         return _ISIS_RandId("**.**.**.*-*")
@@ -206,33 +228,197 @@ class ISIS_LspIdField(_ISIS_IdFieldBase):
 class ISIS_CircuitTypeField(FlagsField):
     def __init__(self, name="circuittype", default=2, size=8,
                  names=None):
-        FlagsField.__init__(self, name, default, size, names)
         if names is None:
             names = ["L1", "L2", "r0", "r1", "r2", "r3", "r4", "r5"]
+        FlagsField.__init__(self, name, default, size, names)
+
+
+def _ISIS_GuessTlvClass_Helper(tlv_classes, defaultname, p, **kargs):
+    cls = conf.raw_layer
+    if len(p) >= 2:
+        tlvtype = orb(p[0])
+        clsname = tlv_classes.get(tlvtype, defaultname)
+        cls = globals()[clsname]
+
+    return cls(p, **kargs)
+
+
+class _ISIS_GenericTlv_Base(Packet):
+    fields_desc = [ByteField("type", 0),
+                   FieldLenField("len", None, length_of="val", fmt="B"),
+                   BoundStrLenField("val", "", length_from=lambda pkt: pkt.len)]  # noqa: E501
+
+    def guess_payload_class(self, p):
+        return conf.padding_layer
+
+
+class ISIS_GenericTlv(_ISIS_GenericTlv_Base):
+    name = "ISIS Generic TLV"
+
+
+class ISIS_GenericSubTlv(_ISIS_GenericTlv_Base):
+    name = "ISIS Generic Sub-TLV"
 
 
 #######################################################################
-##  ISIS TLVs                                                        ##
+#   ISIS Sub-TLVs for TLVs 22, 23, 141, 222, 223                      #
 #######################################################################
-_isis_tlv_classes = { 
+_isis_subtlv_classes_1 = {
+    3: "ISIS_AdministrativeGroupSubTlv",
+    4: "ISIS_LinkLocalRemoteIdentifiersSubTlv",
+    6: "ISIS_IPv4InterfaceAddressSubTlv",
+    8: "ISIS_IPv4NeighborAddressSubTlv",
+    9: "ISIS_MaximumLinkBandwidthSubTlv",
+    10: "ISIS_MaximumReservableLinkBandwidthSubTlv",
+    11: "ISIS_UnreservedBandwidthSubTlv",
+    12: "ISIS_IPv6InterfaceAddressSubTlv",
+    13: "ISIS_IPv6NeighborAddressSubTlv",
+    18: "ISIS_TEDefaultMetricSubTlv"
+}
+
+_isis_subtlv_names_1 = {
+    3: "Administrative Group (Color)",
+    4: "Link Local/Remote Identifiers",
+    6: "IPv4 Interface Address",
+    8: "IPv4 Neighbor Address",
+    9: "Maximum Link Bandwidth",
+    10: "Maximum Reservable Link Bandwidth",
+    11: "Unreserved Bandwidth",
+    12: "IPv6 Interface Address",
+    13: "IPv6 Neighbor Address",
+    18: "TE Default Metric"
+}
+
+
+def _ISIS_GuessSubTlvClass_1(p, **kargs):
+    return _ISIS_GuessTlvClass_Helper(_isis_subtlv_classes_1, "ISIS_GenericSubTlv", p, **kargs)  # noqa: E501
+
+
+class ISIS_IPv4InterfaceAddressSubTlv(ISIS_GenericSubTlv):
+    name = "ISIS IPv4 Interface Address (S)"
+    fields_desc = [ByteEnumField("type", 6, _isis_subtlv_names_1),
+                   FieldLenField("len", None, length_of="address", fmt="B"),
+                   IPField("address", "0.0.0.0")]
+
+
+class ISIS_IPv4NeighborAddressSubTlv(ISIS_GenericSubTlv):
+    name = "ISIS IPv4 Neighbor Address (S)"
+    fields_desc = [ByteEnumField("type", 8, _isis_subtlv_names_1),
+                   FieldLenField("len", None, length_of="address", fmt="B"),
+                   IPField("address", "0.0.0.0")]
+
+
+class ISIS_LinkLocalRemoteIdentifiersSubTlv(ISIS_GenericSubTlv):
+    name = "ISIS Link Local/Remote Identifiers (S)"
+    fields_desc = [ByteEnumField("type", 4, _isis_subtlv_names_1),
+                   FieldLenField("len", 8, fmt="B"),
+                   IntField("localid", "0"),
+                   IntField("remoteid", "0")]
+
+
+class ISIS_IPv6InterfaceAddressSubTlv(ISIS_GenericSubTlv):
+    name = "ISIS IPv6 Interface Address (S)"
+    fields_desc = [ByteEnumField("type", 12, _isis_subtlv_names_1),
+                   FieldLenField("len", None, length_of="address", fmt="B"),
+                   IP6Field("address", "::")]
+
+
+class ISIS_IPv6NeighborAddressSubTlv(ISIS_GenericSubTlv):
+    name = "ISIS IPv6 Neighbor Address (S)"
+    fields_desc = [ByteEnumField("type", 13, _isis_subtlv_names_1),
+                   FieldLenField("len", None, length_of="address", fmt="B"),
+                   IP6Field("address", "::")]
+
+
+class ISIS_AdministrativeGroupSubTlv(ISIS_GenericSubTlv):
+    name = "Administrative Group SubTLV (Color)"
+    fields_desc = [ByteEnumField("code", 3, _isis_subtlv_names_1),
+                   FieldLenField("len", None, length_of="admingroup", fmt="B"),
+                   IPField("admingroup", "0.0.0.1")]
+
+
+class ISIS_MaximumLinkBandwidthSubTlv(ISIS_GenericSubTlv):
+    name = "Maximum Link Bandwidth SubTLV"
+    fields_desc = [ByteEnumField("type", 9, _isis_subtlv_names_1),
+                   FieldLenField("len", None, length_of="maxbw", fmt="B"),
+                   IEEEFloatField("maxbw", 1000)]  # in B/s
+
+
+class ISIS_MaximumReservableLinkBandwidthSubTlv(ISIS_GenericSubTlv):
+    name = "Maximum Reservable Link Bandwidth SubTLV"
+    fields_desc = [ByteEnumField("type", 10, _isis_subtlv_names_1),
+                   FieldLenField("len", None, length_of="maxrsvbw", fmt="B"),
+                   IEEEFloatField("maxrsvbw", 1000)]  # in B/s
+
+
+class ISIS_UnreservedBandwidthSubTlv(ISIS_GenericSubTlv):
+    name = "Unreserved Bandwidth SubTLV"
+    fields_desc = [ByteEnumField("type", 11, _isis_subtlv_names_1),
+                   FieldLenField("len", None, length_of="unrsvbw", fmt="B"),
+                   FieldListField("unrsvbw", [1000, 1000, 1000, 1000, 1000, 1000, 1000, 1000], IEEEFloatField("", 1000), count_from=lambda pkt: pkt.len / 4)]  # in B/s  # noqa: E501
+
+
+class ISIS_TEDefaultMetricSubTlv(ISIS_GenericSubTlv):
+    name = "TE Default Metric SubTLV"
+    fields_desc = [ByteEnumField("type", 18, _isis_subtlv_names_1),
+                   FieldLenField("len", None, length_of="temetric", adjust=lambda pkt, x:x - 1, fmt="B"),  # noqa: E501
+                   ThreeBytesField("temetric", 1000)]
+
+
+#######################################################################
+#   ISIS Sub-TLVs for TLVs 135, 235, 236, and 237                     #
+#######################################################################
+_isis_subtlv_classes_2 = {
+    1: "ISIS_32bitAdministrativeTagSubTlv",
+    2: "ISIS_64bitAdministrativeTagSubTlv"
+}
+
+_isis_subtlv_names_2 = {
+    1: "32-bit Administrative Tag",
+    2: "64-bit Administrative Tag"
+}
+
+
+def _ISIS_GuessSubTlvClass_2(p, **kargs):
+    return _ISIS_GuessTlvClass_Helper(_isis_subtlv_classes_2, "ISIS_GenericSubTlv", p, **kargs)  # noqa: E501
+
+
+class ISIS_32bitAdministrativeTagSubTlv(ISIS_GenericSubTlv):
+    name = "ISIS 32-bit Administrative Tag (S)"
+    fields_desc = [ByteEnumField("type", 1, _isis_subtlv_names_2),
+                   FieldLenField("len", None, length_of="tags", fmt="B"),
+                   FieldListField("tags", [], IntField("", 0), count_from=lambda pkt: pkt.len // 4)]  # noqa: E501
+
+
+class ISIS_64bitAdministrativeTagSubTlv(ISIS_GenericSubTlv):
+    name = "ISIS 64-bit Administrative Tag (S)"
+    fields_desc = [ByteEnumField("type", 2, _isis_subtlv_names_2),
+                   FieldLenField("len", None, length_of="tags", fmt="B"),
+                   FieldListField("tags", [], LongField("", 0), count_from=lambda pkt: pkt.len // 8)]  # noqa: E501
+
+
+#######################################################################
+#   ISIS TLVs                                                         #
+#######################################################################
+_isis_tlv_classes = {
     1: "ISIS_AreaTlv",
     2: "ISIS_IsReachabilityTlv",
     6: "ISIS_IsNeighbourTlv",
     8: "ISIS_PaddingTlv",
     9: "ISIS_LspEntryTlv",
-   10: "ISIS_AuthenticationTlv",
-   12: "ISIS_ChecksumTlv",
-   14: "ISIS_BufferSizeTlv",
-   22: "ISIS_ExtendedIsReachabilityTlv",
-  128: "ISIS_InternalIpReachabilityTlv",
-  129: "ISIS_ProtocolsSupportedTlv",
-  130: "ISIS_ExternalIpReachabilityTlv",
-  132: "ISIS_IpInterfaceAddressTlv",
-  135: "ISIS_ExtendedIpReachabilityTlv",
-  137: "ISIS_DynamicHostnameTlv",
-  232: "ISIS_Ipv6InterfaceAddressTlv",
-  236: "ISIS_Ipv6ReachabilityTlv",
-  240: "ISIS_P2PAdjacencyStateTlv"
+    10: "ISIS_AuthenticationTlv",
+    12: "ISIS_ChecksumTlv",
+    14: "ISIS_BufferSizeTlv",
+    22: "ISIS_ExtendedIsReachabilityTlv",
+    128: "ISIS_InternalIpReachabilityTlv",
+    129: "ISIS_ProtocolsSupportedTlv",
+    130: "ISIS_ExternalIpReachabilityTlv",
+    132: "ISIS_IpInterfaceAddressTlv",
+    135: "ISIS_ExtendedIpReachabilityTlv",
+    137: "ISIS_DynamicHostnameTlv",
+    232: "ISIS_Ipv6InterfaceAddressTlv",
+    236: "ISIS_Ipv6ReachabilityTlv",
+    240: "ISIS_P2PAdjacencyStateTlv"
 }
 
 _isis_tlv_names = {
@@ -242,70 +428,54 @@ _isis_tlv_names = {
     7: "Instance Identifier TLV",
     8: "Padding TLV",
     9: "LSP Entries TLV",
-   10: "Authentication TLV",
-   12: "Optional Checksum TLV",
-   13: "Purge Originator Identification TLV", 
-   14: "LSP Buffer Size TLV",
-   22: "Extended IS-Reachability TLV",
-   23: "IS Neighbour Attribute TLV",
-   24: "IS Alias ID",
-  128: "IP Internal Reachability TLV",
-  129: "Protocols Supported TLV",
-  130: "IP External Reachability TLV",
-  131: "Inter-Domain Routing Protocol Information TLV",
-  132: "IP Interface Address TLV",
-  134: "Traffic Engineering Router ID TLV",
-  135: "Extended IP Reachability TLV",
-  137: "Dynamic Hostname TLV",
-  138: "GMPLS Shared Risk Link Group TLV",
-  139: "IPv6 Shared Risk Link Group TLV",
-  140: "IPv6 Traffic Engineering Router ID TLV",
-  141: "Inter-AS Reachability Information TLV",
-  142: "Group Address TLV",
-  143: "Multi-Topology-Aware Port Capability TLV",
-  144: "Multi-Topology Capability TLV",
-  145: "TRILL Neighbour TLV",
-  147: "MAC-Reachability TLV",
-  148: "BFD-Enabled TLV",
-  211: "Restart TLV",
-  222: "Multi-Topology Intermediate Systems TLV",
-  223: "Multi-Topology IS Neighbour Attributes TLV",
-  229: "Multi-Topology TLV",
-  232: "IPv6 Interface Address TLV",
-  233: "IPv6 Global Interface Address TLV",
-  235: "Multi-Topology IPv4 Reachability TLV",
-  236: "IPv6 Reachability TLV",
-  237: "Multi-Topology IPv6 Reachability TLV",
-  240: "Point-to-Point Three-Way Adjacency TLV",
-  242: "IS-IS Router Capability TLV",
-  251: "Generic Information TLV"
+    10: "Authentication TLV",
+    12: "Optional Checksum TLV",
+    13: "Purge Originator Identification TLV",
+    14: "LSP Buffer Size TLV",
+    22: "Extended IS-Reachability TLV",
+    23: "IS Neighbour Attribute TLV",
+    24: "IS Alias ID",
+    128: "IP Internal Reachability TLV",
+    129: "Protocols Supported TLV",
+    130: "IP External Reachability TLV",
+    131: "Inter-Domain Routing Protocol Information TLV",
+    132: "IP Interface Address TLV",
+    134: "Traffic Engineering Router ID TLV",
+    135: "Extended IP Reachability TLV",
+    137: "Dynamic Hostname TLV",
+    138: "GMPLS Shared Risk Link Group TLV",
+    139: "IPv6 Shared Risk Link Group TLV",
+    140: "IPv6 Traffic Engineering Router ID TLV",
+    141: "Inter-AS Reachability Information TLV",
+    142: "Group Address TLV",
+    143: "Multi-Topology-Aware Port Capability TLV",
+    144: "Multi-Topology Capability TLV",
+    145: "TRILL Neighbour TLV",
+    147: "MAC-Reachability TLV",
+    148: "BFD-Enabled TLV",
+    211: "Restart TLV",
+    222: "Multi-Topology Intermediate Systems TLV",
+    223: "Multi-Topology IS Neighbour Attributes TLV",
+    229: "Multi-Topology TLV",
+    232: "IPv6 Interface Address TLV",
+    233: "IPv6 Global Interface Address TLV",
+    235: "Multi-Topology IPv4 Reachability TLV",
+    236: "IPv6 Reachability TLV",
+    237: "Multi-Topology IPv6 Reachability TLV",
+    240: "Point-to-Point Three-Way Adjacency TLV",
+    242: "IS-IS Router Capability TLV",
+    251: "Generic Information TLV"
 }
 
 
 def _ISIS_GuessTlvClass(p, **kargs):
-    cls = conf.raw_layer
-    if len(p) >= 2:
-        tlvtype = struct.unpack("!B", p[0])[0]
-        clsname = _isis_tlv_classes.get(tlvtype, "ISIS_GenericTlv")
-        cls = globals()[clsname]
-
-    return cls(p, **kargs)
-
-
-class ISIS_GenericTlv(Packet):
-    name = "ISIS Generic TLV"
-    fields_desc = [ByteEnumField("type", 0, _isis_tlv_names),
-                   FieldLenField("len", None, length_of="val", fmt="B"),
-                   BoundStrLenField("val", "", length_from=lambda pkt: pkt.len)]
-
-    def guess_payload_class(self, p):
-        return conf.padding_layer
+    return _ISIS_GuessTlvClass_Helper(_isis_tlv_classes, "ISIS_GenericTlv", p, **kargs)  # noqa: E501
 
 
 class ISIS_AreaEntry(Packet):
     name = "ISIS Area Entry"
     fields_desc = [FieldLenField("arealen", None, length_of="areaid", fmt="B"),
-                   ISIS_AreaIdField("areaid", "49", length_from=lambda pkt: pkt.arealen)]
+                   ISIS_AreaIdField("areaid", "49", length_from=lambda pkt: pkt.arealen)]  # noqa: E501
 
     def extract_padding(self, s):
         return "", s
@@ -314,16 +484,16 @@ class ISIS_AreaEntry(Packet):
 class ISIS_AreaTlv(ISIS_GenericTlv):
     name = "ISIS Area TLV"
     fields_desc = [ByteEnumField("type", 1, _isis_tlv_names),
-                   FieldLenField("len", None, length_of= "areas", fmt="B"),
-                   PacketListField("areas", [], ISIS_AreaEntry, length_from=lambda x: x.len)]
+                   FieldLenField("len", None, length_of="areas", fmt="B"),
+                   PacketListField("areas", [], ISIS_AreaEntry, length_from=lambda x: x.len)]  # noqa: E501
 
 
 class ISIS_AuthenticationTlv(ISIS_GenericTlv):
     name = "ISIS Authentication TLV"
     fields_desc = [ByteEnumField("type", 10, _isis_tlv_names),
-                   FieldLenField("len", None, length_of= "password", adjust=lambda pkt,x: x + 1, fmt="B"),
+                   FieldLenField("len", None, length_of="password", adjust=lambda pkt, x: x + 1, fmt="B"),  # noqa: E501
                    ByteEnumField("authtype", 1, {1: "Plain", 17: "HMAC-MD5"}),
-                   BoundStrLenField("password", "", maxlen= 254, length_from=lambda pkt: pkt.len - 1)]
+                   BoundStrLenField("password", "", maxlen=254, length_from=lambda pkt: pkt.len - 1)]  # noqa: E501
 
 
 class ISIS_BufferSizeTlv(ISIS_GenericTlv):
@@ -343,22 +513,8 @@ class ISIS_ChecksumTlv(ISIS_GenericTlv):
 class ISIS_DynamicHostnameTlv(ISIS_GenericTlv):
     name = "ISIS Dynamic Hostname TLV"
     fields_desc = [ByteEnumField("type", 137, _isis_tlv_names),
-                   FieldLenField("len", None, length_of= "hostname", fmt="B"),
-                   BoundStrLenField("hostname", "", length_from=lambda pkt: pkt.len)]
-
-
-class ISIS_GenericSubTlv(Packet):
-    name = "ISIS Generic Sub-TLV"
-    fields_desc = [ByteField("type", 0),
-                   FieldLenField("len", None, length_of="val", fmt="B"),
-                   BoundStrLenField("val", "", length_from=lambda pkt: pkt.len)]
-
-    def guess_payload_class(self, p):
-        return conf.padding_layer
-
-
-def _isis_guess_subtlv_cls(p, **kargs):
-    return ISIS_GenericSubTlv(p, **kargs)
+                   FieldLenField("len", None, length_of="hostname", fmt="B"),
+                   BoundStrLenField("hostname", "", length_from=lambda pkt: pkt.len)]  # noqa: E501
 
 
 class ISIS_ExtendedIpPrefix(Packet):
@@ -368,45 +524,54 @@ class ISIS_ExtendedIpPrefix(Packet):
         BitField("updown", 0, 1),
         BitField("subtlvindicator", 0, 1),
         BitFieldLenField("pfxlen", None, 6, length_of="pfx"),
-        IPPrefixField("pfx", None, wordbytes=1, length_from=lambda x: x.pfxlen),
-        ConditionalField(FieldLenField("subtlvslen", None, length_of=lambda x: x.subtlvs, fmt= "B"), lambda pkt: pkt.subtlvindicator == 1), 
-        ConditionalField(PacketListField("subtlvs", [], _isis_guess_subtlv_cls, length_from=lambda x: x.subtlvslen), lambda pkt: pkt.subtlvindicator == 1)
+        IPPrefixField("pfx", None, wordbytes=1, length_from=lambda x: x.pfxlen),  # noqa: E501
+        ConditionalField(FieldLenField("subtlvslen", None, length_of="subtlvs", fmt="B"), lambda pkt: pkt.subtlvindicator == 1),  # noqa: E501
+        ConditionalField(PacketListField("subtlvs", [], _ISIS_GuessSubTlvClass_2, length_from=lambda x: x.subtlvslen), lambda pkt: pkt.subtlvindicator == 1)  # noqa: E501
     ]
 
     def extract_padding(self, s):
         return "", s
 
- 
+
+class ISIS_TERouterIDTlv(ISIS_GenericTlv):
+    name = "ISIS TE Router ID TLV"
+    fields_desc = [ByteEnumField("type", 134, _isis_tlv_names),
+                   FieldLenField("len", None, length_of="routerid", fmt="B"),
+                   IPField("routerid", "0.0.0.0")]
+
+
 class ISIS_ExtendedIpReachabilityTlv(ISIS_GenericTlv):
     name = "ISIS Extended IP Reachability TLV"
     fields_desc = [ByteEnumField("type", 135, _isis_tlv_names),
                    FieldLenField("len", None, length_of="pfxs", fmt="B"),
-                   PacketListField("pfxs", [], ISIS_ExtendedIpPrefix, length_from= lambda pkt: pkt.len)]
+                   PacketListField("pfxs", [], ISIS_ExtendedIpPrefix, length_from=lambda pkt: pkt.len)]  # noqa: E501
 
 
 class ISIS_ExtendedIsNeighbourEntry(Packet):
     name = "ISIS Extended IS Neighbour Entry"
-    fields_desc = [ISIS_NodeIdField("neighbourid", "0102.0304.0506.07"),
-                   ThreeBytesField("metric", 1),
-                   FieldLenField("subtlvslen", None, length_of="subtlvs", fmt= "B"),
-                   ConditionalField(PacketListField("subtlvs", [], _isis_guess_subtlv_cls, length_from=lambda x: x.subtlvslen), lambda pkt: pkt.subtlvslen > 0)]
+    fields_desc = [
+        ISIS_NodeIdField("neighbourid", "0102.0304.0506.07"),
+        ThreeBytesField("metric", 1),
+        FieldLenField("subtlvslen", None, length_of="subtlvs", fmt="B"),
+        PacketListField("subtlvs", [], _ISIS_GuessSubTlvClass_1, length_from=lambda x: x.subtlvslen)  # noqa: E501
+    ]
 
     def extract_padding(self, s):
         return "", s
 
-    
+
 class ISIS_ExtendedIsReachabilityTlv(ISIS_GenericTlv):
     name = "ISIS Extended IS Reachability TLV"
     fields_desc = [ByteEnumField("type", 22, _isis_tlv_names),
                    FieldLenField("len", None, length_of="neighbours", fmt="B"),
-                   PacketListField("neighbours", [], ISIS_ExtendedIsNeighbourEntry, length_from=lambda x: x.len)]
+                   PacketListField("neighbours", [], ISIS_ExtendedIsNeighbourEntry, length_from=lambda x: x.len)]  # noqa: E501
 
 
 class ISIS_IpInterfaceAddressTlv(ISIS_GenericTlv):
     name = "ISIS IP Interface Address TLV"
     fields_desc = [ByteEnumField("type", 132, _isis_tlv_names),
-                   FieldLenField("len", None, length_of= "addresses", fmt="B"),
-                   FieldListField("addresses", [], IPField("", "0.0.0.0"), count_from= lambda pkt: pkt.len / 4)]
+                   FieldLenField("len", None, length_of="addresses", fmt="B"),
+                   FieldListField("addresses", [], IPField("", "0.0.0.0"), count_from=lambda pkt: pkt.len // 4)]  # noqa: E501
 
 
 class ISIS_Ipv6InterfaceAddressTlv(ISIS_GenericTlv):
@@ -414,7 +579,7 @@ class ISIS_Ipv6InterfaceAddressTlv(ISIS_GenericTlv):
     fields_desc = [
         ByteEnumField("type", 232, _isis_tlv_names),
         FieldLenField("len", None, length_of="addresses", fmt="B"),
-        IP6ListField("addresses", [], count_from=lambda pkt: pkt.len / 16)
+        IP6ListField("addresses", [], count_from=lambda pkt: pkt.len // 16)
     ]
 
 
@@ -427,9 +592,9 @@ class ISIS_Ipv6Prefix(Packet):
         BitField("subtlvindicator", 0, 1),
         BitField("reserved", 0, 5),
         FieldLenField("pfxlen", None, length_of="pfx", fmt="B"),
-        IP6PrefixField("pfx", None, wordbytes=1, length_from=lambda x: x.pfxlen),
-        ConditionalField(FieldLenField("subtlvslen", None, length_of=lambda x: x.subtlvs, fmt= "B"), lambda pkt: pkt.subtlvindicator == 1), 
-        ConditionalField(PacketListField("subtlvs", [], _isis_guess_subtlv_cls, length_from=lambda x: x.subtlvslen), lambda pkt: pkt.subtlvindicator == 1)
+        IP6PrefixField("pfx", None, wordbytes=1, length_from=lambda x: x.pfxlen),  # noqa: E501
+        ConditionalField(FieldLenField("subtlvslen", None, length_of="subtlvs", fmt="B"), lambda pkt: pkt.subtlvindicator == 1),  # noqa: E501
+        ConditionalField(PacketListField("subtlvs", [], _ISIS_GuessSubTlvClass_2, length_from=lambda x: x.subtlvslen), lambda pkt: pkt.subtlvindicator == 1)  # noqa: E501
     ]
 
     def extract_padding(self, s):
@@ -437,17 +602,17 @@ class ISIS_Ipv6Prefix(Packet):
 
 
 class ISIS_Ipv6ReachabilityTlv(ISIS_GenericTlv):
-    name= "ISIS IPv6 Reachability TLV"
+    name = "ISIS IPv6 Reachability TLV"
     fields_desc = [ByteEnumField("type", 236, _isis_tlv_names),
-                   FieldLenField("len", None, length_of= "pfxs", fmt="B"),
-                   PacketListField("pfxs", [], ISIS_Ipv6Prefix, length_from= lambda pkt: pkt.len)]
+                   FieldLenField("len", None, length_of="pfxs", fmt="B"),
+                   PacketListField("pfxs", [], ISIS_Ipv6Prefix, length_from=lambda pkt: pkt.len)]  # noqa: E501
 
 
 class ISIS_IsNeighbourTlv(ISIS_GenericTlv):
     name = "ISIS IS Neighbour TLV"
     fields_desc = [ByteEnumField("type", 6, _isis_tlv_names),
-                   FieldLenField("len", None, length_of= "neighbours", fmt="B"),
-                   FieldListField("neighbours", [], MACField("", "00.00.00.00.00.00"), count_from= lambda pkt: pkt.len / 6)]
+                   FieldLenField("len", None, length_of="neighbours", fmt="B"),
+                   FieldListField("neighbours", [], MACField("", "00.00.00.00.00.00"), count_from=lambda pkt: pkt.len // 6)]  # noqa: E501
 
 
 class ISIS_LspEntry(Packet):
@@ -466,7 +631,7 @@ class ISIS_LspEntryTlv(ISIS_GenericTlv):
     fields_desc = [
         ByteEnumField("type", 9, _isis_tlv_names),
         FieldLenField("len", None, length_of="entries", fmt="B"),
-        PacketListField("entries", [], ISIS_LspEntry, count_from=lambda pkt: pkt.len / 16)
+        PacketListField("entries", [], ISIS_LspEntry, count_from=lambda pkt: pkt.len // 16)  # noqa: E501
     ]
 
 
@@ -487,11 +652,11 @@ class _AdjacencyStateTlvLenField(Field):
 class ISIS_P2PAdjacencyStateTlv(ISIS_GenericTlv):
     name = "ISIS P2P Adjacency State TLV"
     fields_desc = [ByteEnumField("type", 240, _isis_tlv_names),
-               _AdjacencyStateTlvLenField("len", None, fmt="B"),
-               ByteEnumField("state", "Down", {0x2 : "Down", 0x1 : "Initialising", 0x0 : "Up"}),
-               ConditionalField(IntField("extlocalcircuitid", None), lambda pkt: pkt.len >= 5),
-               ConditionalField(ISIS_SystemIdField("neighboursystemid", None), lambda pkt: pkt.len >= 11),
-               ConditionalField(IntField("neighbourextlocalcircuitid", None), lambda pkt: pkt.len == 15)]
+                   _AdjacencyStateTlvLenField("len", None, fmt="B"),
+                   ByteEnumField("state", "Down", {0x2: "Down", 0x1: "Initialising", 0x0: "Up"}),  # noqa: E501
+                   ConditionalField(IntField("extlocalcircuitid", None), lambda pkt: pkt.len >= 5),  # noqa: E501
+                   ConditionalField(ISIS_SystemIdField("neighboursystemid", None), lambda pkt: pkt.len >= 11),  # noqa: E501
+                   ConditionalField(IntField("neighbourextlocalcircuitid", None), lambda pkt: pkt.len == 15)]  # noqa: E501
 
 
 # TODO dynamically allocate sufficient size
@@ -509,12 +674,12 @@ class ISIS_ProtocolsSupportedTlv(ISIS_GenericTlv):
     fields_desc = [
         ByteEnumField("type", 129, _isis_tlv_names),
         FieldLenField("len", None, count_of="nlpids", fmt="B"),
-        FieldListField("nlpids", [], ByteEnumField("", "IPv4", network_layer_protocol_ids), count_from=lambda pkt: pkt.len)
+        FieldListField("nlpids", [], ByteEnumField("", "IPv4", network_layer_protocol_ids), count_from=lambda pkt: pkt.len)  # noqa: E501
     ]
 
 
 #######################################################################
-##  ISIS Old-Style TLVs                                              ##
+#   ISIS Old-Style TLVs                                               #
 #######################################################################
 
 class ISIS_IpReachabilityEntry(Packet):
@@ -535,16 +700,16 @@ class ISIS_InternalIpReachabilityTlv(ISIS_GenericTlv):
     fields_desc = [
         ByteEnumField("type", 128, _isis_tlv_names),
         FieldLenField("len", None, length_of="entries", fmt="B"),
-        PacketListField("entries", [], ISIS_IpReachabilityEntry, count_from=lambda x: x.len / 12)
+        PacketListField("entries", [], ISIS_IpReachabilityEntry, count_from=lambda x: x.len // 12)  # noqa: E501
     ]
 
 
-class ISIS_ExternalIpReachabilityTLV(ISIS_GenericTlv):
+class ISIS_ExternalIpReachabilityTlv(ISIS_GenericTlv):
     name = "ISIS External IP Reachability TLV"
     fields_desc = [
         ByteEnumField("type", 130, _isis_tlv_names),
         FieldLenField("len", None, length_of="entries", fmt="B"),
-        PacketListField("entries", [], ISIS_IpReachabilityEntry, count_from=lambda x: x.len / 12)
+        PacketListField("entries", [], ISIS_IpReachabilityEntry, count_from=lambda x: x.len // 12)  # noqa: E501
     ]
 
 
@@ -564,13 +729,14 @@ class ISIS_IsReachabilityTlv(ISIS_GenericTlv):
     name = "ISIS IS Reachability TLV"
     fields_desc = [
         ByteEnumField("type", 2, _isis_tlv_names),
-        FieldLenField("len", None, fmt="B", length_of="neighbours", adjust=lambda pkt,x: x+1),
+        FieldLenField("len", None, fmt="B", length_of="neighbours", adjust=lambda pkt, x: x + 1),  # noqa: E501
         ByteField("virtual", 0),
-        PacketListField("neighbours", [], ISIS_IsReachabilityEntry, count_from=lambda x: (x.len - 1) / 11)
+        PacketListField("neighbours", [], ISIS_IsReachabilityEntry, count_from=lambda x: (x.len - 1) // 11)  # noqa: E501
     ]
 
+
 #######################################################################
-##  ISIS PDU Packets                                                 ##
+#   ISIS PDU Packets                                                  #
 #######################################################################
 _isis_pdu_names = {
     15: "L1 LAN Hello",
@@ -606,7 +772,7 @@ class ISIS_CommonHdr(Packet):
         if checksumInfo is not None:
             (cbegin, cpos) = checksumInfo
             checkbytes = fletcher16_checkbytes(pdu[cbegin:], (cpos - cbegin))
-            pdu = pdu[:cpos] + checkbytes + pdu[cpos+2:]
+            pdu = pdu[:cpos] + checkbytes + pdu[cpos + 2:]
 
         return pdu
 
@@ -629,12 +795,12 @@ class _ISIS_PduBase(Packet):
 
 class _ISIS_PduLengthField(FieldLenField):
     def __init__(self):
-        FieldLenField.__init__(self, "pdulength", None, length_of="tlvs", adjust=lambda pkt,x: x + pkt.underlayer.hdrlen)
+        FieldLenField.__init__(self, "pdulength", None, length_of="tlvs", adjust=lambda pkt, x: x + pkt.underlayer.hdrlen)  # noqa: E501
 
 
 class _ISIS_TlvListField(PacketListField):
     def __init__(self):
-        PacketListField.__init__(self, "tlvs", [], _ISIS_GuessTlvClass, count_from= None, length_from= lambda pkt: pkt.pdulength - pkt.underlayer.hdrlen)
+        PacketListField.__init__(self, "tlvs", [], _ISIS_GuessTlvClass, length_from=lambda pkt: pkt.pdulength - pkt.underlayer.hdrlen)  # noqa: E501
 
 
 class _ISIS_LAN_HelloBase(_ISIS_PduBase):
@@ -677,7 +843,7 @@ class _ISIS_LSP_Base(_ISIS_PduBase):
         ISIS_LspIdField("lspid", "0102.0304.0506.00-00"),
         XIntField("seqnum", 0x00000001),
         XShortField("checksum", None),
-        FlagsField("typeblock", 0x03, 8, ["L1", "L2", "OL", "ADef", "ADel", "AExp", "AErr", "P"]),
+        FlagsField("typeblock", 0x03, 8, ["L1", "L2", "OL", "ADef", "ADel", "AExp", "AErr", "P"]),  # noqa: E501
         _ISIS_TlvListField()
     ]
 
@@ -757,6 +923,7 @@ class ISIS_L2_PSNP(_ISIS_PSNP_Base):
     def answers(self, other):
         return _snp_answers(self, other, "ISIS_L2_LSP")
 
+
 register_cln_protocol(0x83, ISIS_CommonHdr)
 bind_layers(ISIS_CommonHdr, ISIS_L1_LAN_Hello, hdrlen=27, pdutype=15)
 bind_layers(ISIS_CommonHdr, ISIS_L2_LAN_Hello, hdrlen=27, pdutype=16)
